#################################################################
# Function: Semiparametric Sample Selection model               #
#                                                               #
# A function for fitting semiparametric sample selection models #
# with possible geoadditive effects and varying coefficients.   #
# Inference is fully Bayesian.                                  #
#                                                               #
# Author: Manuel Wiesenfarth, first version 04.08.2008          #
#                                                               #
#                                                               #
#################################################################


##############################################################
#                                                            #
# Requires packages mvtnorm and splines. Package BayesX 0.2  #
# is required if spatial effects are included.               #
# Comparative estimation with package mgcv and package       #
# sampleSelection is possible                                #
#                                                            #
##############################################################

###################################################################################
#
# selection, outcome: The formulas of selection and outcome equation, respectively.
#                     Specification of formulas is similar
#                     to the mgcv package, with an additional possibility to fit spatial
#                     effects specified in the formula with sp(.) (smooth functions
#                     are specified with s(.) and varying coefficients with
#                     s(., by=.)). If nonparametric effects are included, an
#                     intercept is obligatory, otherwise exclusion of the intercept with
#                     -1 and +0, respectively, as usual.
# data: A data frame containing the variables in the model.
# map.sel, map.out: Specification either of a
#                   .bnd map file or of a bnd map object for spatial effects in the
#                   selection and outcome equation, respectively. If both equations include a spatial effect and only one map is
#                   specified, maps are assumed to be equal in both equations.
# graph.sel, graph.out: Specification of '.gra' graph files for spatial effects in the selection and outcome
#                       equation, respectively, containing information of neighborhood structures. If only one graph file is specified,
#                       neighborhood structures are assumed to be equal in both equations.
# numBurnIn, numSamples: Number of iterations in burn-in period that will be discarded and number of samples that will be used for estimation.
# thin: Specify the degree of thinning. If 1, every sample after the burn-in period is used.
# numKnots: Number of knots in B-spline bases of all nonparametric effects.
# knotchoice: Either "equidistant" or "quantiles".
# degree: Specifies the degree of the B-spline basis functions according to function bs() in package "splines", default is 3 for cubic splines.
# rw.order: Specify the order of the random walk penalty matrices of smooth functions.
# prior.tau: Shape and scale parameters for the inverse Gamma prior on the variance parameters tau2 which influences the smoothness of splines.
# prior.tau.sp: Shape and scale parameters for the inverse Gamma prior on the variance parameters tau2 which influences the smoothness of spatial effects.
# prior.sigma2_1: Shape and scale parameters for the inverse Gamma prior on the covariance component sigma_{2|1}^2 (==sigma2_1).
# sigma2_1.start: Starting value for .... default=10  . Note that all other starting values are set to 0.
# prior.sigma12: Mean and variance of the Gaussian prior on the covariance sigma12 of the errors.
# seed: Specify a seed for set.seed (optional).
# record.npcoef: Record samples of the regression coefficients of nonparametric effects? Returned only if return.samples==TRUE.
# return.samples: Return matrices of samples of all estimates (including components of the covariance matrix, nonparametric (only
#                 if record.npcoef==T) and parametric estimates)?
# return.f: Specify if fitted values for every smooth effect of each Gibbs iteration shall be returned.
# x.pred: (optional) List of vectors of covariates for which predicted function values are returned.
# output: Device on which plots are printed. Default is "window", otherwise pdf files are written with file name starting with what is given here.
# mgcv: Specify if univariate estimation using package mgcv shall be included for comparison. Spatial effects are discarded.
# add.mgcv: Add smooth functions estimated by mgcv to the plots of nonparametric effects. Doesn't work with varying coefficients.
# sampleSelection: Specify if Heckman-style estimation using package sampleSelection is conducted for comparison. Spatial effects are discarded
#                  and nonparametric effects are linearly estimated. Possible values "n" for no estimation, "2step" for Heckman's Two-step estimator,
#                  "ML" for Full Information Maximum Likelihood estimation and "both" for both of them.
# plotacf: Plot autocorrelation functions? If nonparametric effects are included autocorrelation functions of the first and 10th knot are plotted, but only if record.npcoef==T.
# lag: Specify the lag of the autocorrelation functions.
# plotpaths: Logical, determines if TRUE the sampling paths of the parametric effects are plotted
# plotnp: Logical, determines if TRUE nonparametric effects are plotted
# plotmaps: Logical, determines  if TRUE the maps(s) with spatial effect(s) is/are plotted
# cols: Color scheme to be employed for plotting spatial maps. Could be either a vector of colors or one out of the following pre-defined schemes: hcl, hsv, grey
# progressBar: Show txtProgressBar during simulation
#
###################################################################################


ssm =
function(selection, outcome, data=parent.frame(),numBurnIn=2000, numSamples=20000, thin=20
,contrasts.sel =NULL, contrasts.out =NULL
#, map.sel = NULL
, graph.sel= NULL
#, map.out = NULL
, graph.out= NULL
, x.pred=NULL, fix.sigma12=F
, numKnots=20, knotchoice="equidistant", degree=3, rw.order=2
, sigma2_1.start=10
, prior.sigma2_1=c(0.001,0.001), prior.sigma12=c(0,10), prior.tau=c(0.001,0.001), prior.tau.sp=c(0.001,0.001)
, seed=NULL
, record.npcoef=T, return.samples=T, return.f=T
, mgcv=F
#add.mgcv=T,
,sampleSelection="n"
, output="window"
, plotnp=F 
, plotpaths=F
#, plotmaps=T, cols="hcl"
#, plotacf=F,lag=50,
#plotpaths.style="separate"
, progressBar=T
, method="chol",truncnorm=T,prof=F,...){


################################################
# load required libraries
#  if (truncnorm) require(truncnorm)

result=list()

date=format(Sys.time(), "%Y-%m-%d--%Hh%Mm%Ss")
result$date=date

p0=proc.time()
################################################
# Read formulas

  #------------------
  #selection equation
    # Interpret formula of selection equation
      np=interpret.gam(selection)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.sel = terms.formula(selection, specials = c("s", "sp"))
      sp_sel = attr(tf.sel, "specials")$sp
    # Number of nonparametric effects
      numSplines_sel = length(np$smooth.spec)
      numSpatial_sel = length(sp_sel)
    # get smooth parameters and 'by' variables
      npvars.sel= byvars.sel= numeric(numSplines_sel)
      if (numSplines_sel>0){
        for (j in 1:(numSplines_sel)){
          npvars.sel[j]=np$smooth.spec[[j]]$term
          byvars.sel[j]=np$smooth.spec[[j]]$by
        }
      }
    # spatial effects
      spvar.sel=all.vars(selection, unique = FALSE)[-which(all.vars(selection, unique = FALSE) %in% byvars.sel[byvars.sel!="NA"])][sp_sel]

  #--------------------
  # outcome equation

    # interpret formula of outcome equation
      np=interpret.gam(outcome)
    # recognize on which location in the formula there is a s() (for smooth functions) or sp() (for spatial effects)
      tf.out = terms.formula(outcome, specials = c("s", "sp"))
      sp_out = attr(tf.out, "specials")$sp
    # Number of nonparametric effects
      numSplines_out = length(np$smooth.spec)
      numSpatial_out = length(sp_out)
    # get smooth parameters and by variables
      npvars.out= byvars.out= numeric(numSplines_out)
      if (numSplines_out>0){
        for (j in 1:(numSplines_out)){
          npvars.out[j]=np$smooth.spec[[j]]$term
          byvars.out[j]=np$smooth.spec[[j]]$by
        }
      }
    # spatial effects
      spvar.out=all.vars(outcome, unique = FALSE)[-which(all.vars(outcome, unique = FALSE) %in% byvars.out[byvars.out!="NA"])][sp_out]


  #------------------
  # Combined

    # Auxillary variables -> for spatial effects no varying coefficients are implemented, these variables are only needed as wild-cards in 'byvars'
      if (numSpatial_sel>0) byvarSp.sel_aux="NA"  else byvarSp.sel_aux= numeric(0)
      if (numSpatial_out>0) byvarSp.out_aux="NA"  else byvarSp.out_aux= numeric(0)

    # Vector of nonparametric effects including spatial effects if included  (note the order of selection&outcome&spatial!)
      npvars=c(npvars.sel,spvar.sel,npvars.out,spvar.out)
      byvars=c(byvars.sel, byvarSp.sel_aux, byvars.out, byvarSp.out_aux)    


################################################
# Spatial Data
  # Estimate spatial effects?
    geo=FALSE
#    if (((is.null(map.sel)==F | is.null(map.out)==F) & (is.null(graph.sel)==F | is.null(graph.out)==F)) & (numSpatial_out>0 | numSpatial_sel>0)) geo=TRUE
    if (((is.null(graph.sel)==F | is.null(graph.out)==F)) & (numSpatial_out>0 | numSpatial_sel>0)) geo=TRUE
  # Read spatial data
    # if spatial effects are included and only one graph file is given, the neighborhood structures are assumed to be equal, i.e. graph.sel==graph.out
    if (geo){
      pmat=K.mrf=regions=numRegions=list()
      if ((numSpatial_sel>0&numSpatial_out>0) & (is.null(graph.sel) & !is.null(graph.out))) graph.sel=graph.out
      if ((numSpatial_sel>0&numSpatial_out>0) & (is.null(graph.out) & !is.null(graph.sel))) graph.out=graph.sel
      
      if (numSpatial_sel>0){
        pmat[[1]] =pen.mrf(graph.sel)
        regions[[1]]=pmat[[1]]$regions
        numRegions[[1]]=length(regions[[1]])
        # penalty matrix
          K.mrf[[1]]=pmat[[1]]$K.mrf
      }
      if (numSpatial_out>0){
        pmat[[2]] =pen.mrf(graph.out)
        regions[[2]]=pmat[[2]]$regions
        numRegions[[2]]=length(regions[[2]])
        # penalty matrix
          K.mrf[[2]]=pmat[[2]]$K.mrf
      }
    }


################################################
# Model matrices and responses

  #=============================================================================
  # Responses and model matrices of parametric covariates

    # Model formula for the strictly parametric part of the model.
      selectionPar= read.formula(selection)$pf
      outcomePar= read.formula(outcome)$pf

    # Read formulas and convert to model frames
      frame.selectionPar= model.frame(selectionPar,data=data)
      frame.outcomePar= model.frame(outcomePar, data=data)

    # Responses
      response = list()
      response[[1]] = model.response(frame.selectionPar)
      response[[2]] = model.response(frame.outcomePar)[response[[1]] ==1]
      # Auxillary varible: duplication of response[[1]] (only for sampling of latent vars)
        z_aux= response[[1]]

    # Design matrices of parametric effects
      V = VV0= VV1= list()
      V[[1]] = model.matrix(selectionPar, frame.selectionPar,contrasts.arg=contrasts.sel)
      V[[2]] = model.matrix(outcomePar, frame.outcomePar,contrasts.arg=contrasts.out)[response[[1]]==1, ,drop=F]
    # Auxillary matrices
      VV0[[1]]= crossprod(V[[1]][response[[1]]==0, ,drop=F])
      VV1[[1]]= crossprod(V[[1]][response[[1]]==1, ,drop=F])
      VV1[[2]]= crossprod(V[[2]])

    # Name vector of parametric effects including Intercept if included
      g_sel=dimnames(V[[1]])[[2]]
      g_out=dimnames(V[[2]])[[2]]

  #=============================================================================
  # List of the nonparametric covariates
    npcovariates = lapply(as.list(npvars), function(x) data[[x]])
    byvars.list  = lapply(as.list(byvars), function(x) data[[x]])      # List of variables given by a 'by=' argument for varying coefficients
    if (length(npvars)>0){
      if (numSplines_out>0 | numSpatial_out>0){
        for (j in numSplines_sel+numSpatial_sel+ (1:(numSplines_out+numSpatial_out))){ 
          npcovariates[[j]] = npcovariates[[j]][response[[1]]==1]
          if (byvars[j]!="NA") byvars.list[[j]] = byvars.list[[j]][response[[1]]==1]
        }
      }
      names(npcovariates)=npvars
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(byvars.list)[j] = byvars[j]
    }
  #-----------------------------------------------------------------------------
  # Model matrices of the nonparametric covariates
    X= XX0= XX1= Xby= X.pred=f.pred.sum= list()
 
    if (length(npvars) > 0){
      # Prediction on a given grid if specified
        if (is.null(x.pred)) {
          x.pred=list()
          length(x.pred)=length(npvars)
        }
        else 
        if (!is.list(x.pred)){
            temp=list()
            for (i in 1:length(npvars)) temp[[i]]=x.pred
            x.pred=temp
            rm(temp)
        }

      for (j in 1:length(npvars)){    #first selection, then  outcome equation

        # Write design matrix of spatial covariate
          if (numSpatial_sel>0 & j==numSplines_sel+1){     
            X[[j]] = matrix(0,length(npcovariates[[j]]),numRegions[[1]])
              for(i in 1:numRegions[[1]])
                X[[j]][which(npcovariates[[j]]==regions[[1]][i]),i] = 1     # 1 if y_i1 was observed in region s
          }
          if (numSpatial_out>0 &  j==numSplines_sel+numSpatial_sel+numSplines_out+1){
            X[[j]] = matrix(0,length(npcovariates[[j]]),numRegions[[2]])
              for(i in 1:numRegions[[2]])
                X[[j]][which(npcovariates[[j]]==regions[[2]][i]),i] = 1      # 1 if y_i2 was observed in region s
            if (!is.null(x.pred[[j]])){        
              X.pred[[j]] = matrix(0,length(x.pred[[j]]),numRegions[[2]])
              f.pred.sum[[j]]=0
                for(i in 1:numRegions[[2]])
                  X.pred[[j]][which(x.pred[[j]]==regions[[2]][i]),i] = 1
            }              
          }
        # write B-spline basis design matrix of nonparametric effects into list element j
          if (j<=numSplines_sel | (j>numSplines_sel+numSpatial_sel&j<=numSplines_sel+numSpatial_sel+numSplines_out)){
            X[[j]]=bspline(npcovariates[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
            if (!is.null(x.pred[[j]])){
              X.pred[[j]]=bspline(x.pred[[j]],numKnots=numKnots,knotchoice=knotchoice,degree=degree)
              f.pred.sum[[j]]=0
            }
          }
      
        if (byvars[j]!="NA"){
          Xby[[j]]=X[[j]]
          X[[j]]= diag(byvars.list[[j]]) %*% X[[j]]
        }

        # Auxillary matrices
          if (j <= numSplines_sel+numSpatial_sel){
            #selection eq 
              XX0[[j]]= crossprod(X[[j]][response[[1]]==0, ,drop=F])
              XX1[[j]]= crossprod(X[[j]][response[[1]]==1, ,drop=F])
          }
          else{
            #outcome eq 
              XX1[[j]]= crossprod(X[[j]])
          }
      }
    }
#%%%%%%%%%%%%%%%
# discard unnecessary matrices
  rm(frame.selectionPar, frame.outcomePar)
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

################################################
# Some error search:
  # Intercept: Either 0, indicating no intercept is to be fit, or 1 indicating that an intercept is to be fit
    int_sel = attr(tf.sel,"intercept")
    int_out = attr(tf.out,"intercept")
  if ((int_sel==0 & (numSplines_sel>0|numSpatial_sel>0))|(int_out==0 & (numSplines_out>0|numSpatial_out>0))) stop("When nonparametric effects are included, an intercept is required.")
 # if ((numSpatial_sel>0|numSpatial_out>0) & (!is.list(map.sel)&!is.list(map.out))) stop("No map correctly specified")
  if ((numSpatial_sel>0|numSpatial_out>0) && !geo) stop("Graph not correctly specified")

  if (!all(response[[1]] %in% c(0,1))) stop("Response in selection equation must be coded 0 for unoberseved and 1 for observed")


##############################################################
# Penalty matrix for smooth functions
  D= diff(diag(numKnots+2),differences=rw.order)
  K.rw=crossprod(D)

##############################################################
# Initialization
  if (!is.null(seed)) set.seed(seed)
  cond.var= offset= VAR.gamma= mu.gamma= gamma= f.gamma= eta= list()
  sigma2_1= sigma12= Dev =numeric(numBurnIn+numSamples+1)
  gamma[[1]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[1]])[2])   #dim(V[[1]])[2]=length(gamma[[1]])
  gamma[[2]] = matrix(data=0, nrow=(numBurnIn+numSamples+1), ncol=dim(V[[2]])[2])   #dim(V[[2]])[2]=length(gamma[[2]])
  z.sum = 0

  #***************************************************************************
  # Inizialize betas, f's etc.

    beta= beta.rec= f= f.sum=f.rec= fnr= f.pred=f.pred.rec=list()

    if (length(npvars)>0){
      f[1:length(npvars)]= f.sum[1:length(npvars)]= beta[1:length(npvars)]= 0
      if (!all(unlist(lapply(x.pred,is.null)))) f.pred[1:length(npvars)]= 0
      fnr[1:length(npvars)]=1
      
      for (j in 1:length(npvars)) {
        beta[[j]]= numeric(dim(X[[j]])[2])    #betas have a starting value zero
        if (record.npcoef| byvars[j]!="NA")     #beta is only recorded if desired
          beta.rec[[j]]=matrix(data=0, ncol=dim(X[[j]])[2],nrow=floor(numSamples/thin))
        else beta.rec[[j]]=numeric()
        if (byvars[j]=="NA"){  
          f.rec[[j]]=matrix(data=0, nrow=dim(X[[j]])[1],ncol=floor(numSamples/thin))
          if (!is.null(x.pred[[j]])) f.pred.rec[[j]]=matrix(data=0, nrow=dim(X.pred[[j]])[1],ncol=floor(numSamples/thin))
         }
        else f.rec[[j]]=numeric()
      }
      names(beta.rec)=names(f.rec)=npvars
    }


################################################
# Comperative estimation
  #------------------
  # Auxillary formulas
    #------------------
    # Responses
      response.sel=all.vars(selection)[1]
      response.out=all.vars(outcome)[1]

    #------------------
    # Formulas with all covariates parametric (including those nonparametrically defined)
     fakeformula.sel=as.formula(paste(response.sel,"~",paste(c(all.vars(selectionPar)[-1],npvars.sel), collapse="+")))
      fakeformula.out=as.formula(paste(response.out,"~",paste(c(all.vars(outcomePar)[-1],npvars.out), collapse="+")))

    #------------------
    # Formulas without spatial effects
      covariates.sel=as.vector(attr(tf.sel,"term.labels"))
        if (numSpatial_sel>0) covariates.sel=covariates.sel[-(sp_sel-1)]
      covariates.out=as.vector(attr(tf.out,"term.labels"))
        if (numSpatial_out>0) covariates.out=covariates.out[-(sp_out-1)]

      mgcvformula.sel=as.formula(paste(response.sel,"~",paste(covariates.sel, collapse="+")))
      mgcvformula.out=as.formula(paste(response.out,"~",paste(covariates.out, collapse="+")))

  #------------------
  # Univariate estimation with mgcv package
    result$f.mgcv=result$f.mgcv.pred=list()
    if (mgcv){
      cat("\nUnivariate estimation using package mgcv\n")
   
      mgcv.sel=gam(mgcvformula.sel, data=data, family=binomial(link = "probit"))
      mgcv.out=gam(mgcvformula.out, data=data[data[[response.sel]]==1,])
      print(summary(mgcv.sel))
      print(summary(mgcv.out))

      if (plotnp){
        if (output!="window") pdf(paste(output," mgcv ",date,".pdf",sep=""))# ,version="1.4"
          if (length(mgcv.sel$smooth)>0)
            for (j in 1:length(mgcv.sel$smooth)){
              if (output=="window") dev.new()
              plot(mgcv.sel,select=j,pages=length(mgcv.sel$smooth),shade=T,shade.col= hsv(0.5,0.5,0.5,alpha=0.18))
            }
          if (length(mgcv.out$smooth)>0)
            for (j in 1:length(mgcv.out$smooth)){
              if (output=="window") dev.new()
              plot(mgcv.out,select=j,pages=length(mgcv.out$smooth),shade=T,shade.col= hsv(0.5,0.5,0.5,alpha=0.18))
            }
        if (output!="window") dev.off()
      }

      coef.mgcv_sel=as.vector(mgcv.sel$coefficients[1:ncol(V[[1]])])
      coef.mgcv_out=as.vector(mgcv.out$coefficients[1:ncol(V[[2]])])

      sd.coef.mgcv_sel=sqrt(diag(vcov(mgcv.sel))[1:ncol(V[[1]])])
      sd.coef.mgcv_out=sqrt(diag(vcov(mgcv.out))[1:ncol(V[[2]])])

      # values of f 
        f.mgcv.sel=predict(mgcv.sel,type="terms")
        if (length(attr(mgcv.sel$pterms,"term.labels"))>0) f.mgcv.sel=f.mgcv.sel[,-(1:length(attr(mgcv.sel$pterms,"term.labels"))) ,drop=F]
        f.mgcv.out=predict(mgcv.out,type="terms")
        if (length(attr(mgcv.out$pterms,"term.labels"))>0) f.mgcv.out=f.mgcv.out[,-(1:length(attr(mgcv.out$pterms,"term.labels"))) ,drop=F]
        # as list
          if (numSplines_sel>0) for (j in 1:ncol(f.mgcv.sel)) result$f.mgcv[[j]]=f.mgcv.sel[,j]
          if (numSplines_out>0) for (j in 1:ncol(f.mgcv.out)) result$f.mgcv[[j+ncol(f.mgcv.sel)]]=f.mgcv.out[,j]

        # Prediction on fixed grid
        if ((numSplines_sel>0 | numSplines_out>0) & !all(unlist(lapply(x.pred,is.null)))){
          newdata=V[[1]][1:length(x.pred[[1]]),]
          for (j in 1:numSplines_sel) newdata=as.data.frame(cbind(newdata, x.pred[[j]])) 
          dimnames(newdata)[[2]]=c(g_sel,npvars.sel[1:numSplines_sel])
          pred.mgcv.sel=predict(mgcv.sel,newdata=newdata,type="terms")
          if (length(attr(mgcv.sel$pterms,"term.labels"))>0) pred.mgcv.sel=pred.mgcv.sel[,-(1:length(attr(mgcv.sel$pterms,"term.labels"))) ,drop=F]

          newdata=V[[2]][1:length(x.pred[[numSplines_sel+numSpatial_sel+1]]),]
          for (j in 1:numSplines_out) newdata=as.data.frame(cbind(newdata, x.pred[[numSplines_sel+numSpatial_sel+j]])) 
          dimnames(newdata)[[2]]=c(g_out,npvars.out[1:numSplines_out])
          pred.mgcv.out=predict(mgcv.out,newdata=newdata,type="terms")
          if (length(attr(mgcv.out$pterms,"term.labels"))>0) pred.mgcv.out=pred.mgcv.out[,-(1:length(attr(mgcv.out$pterms,"term.labels"))) ,drop=F]

          # as list
            if (numSplines_sel>0) for (j in 1:ncol(pred.mgcv.sel)) result$f.mgcv.pred[[j]]=pred.mgcv.sel[,j]
            if (numSplines_out>0) for (j in 1:ncol(pred.mgcv.out)) result$f.mgcv.pred[[j+ncol(pred.mgcv.sel)]]=pred.mgcv.out[,j]
        }
      
      rm(mgcv.sel,mgcv.out,f.mgcv.sel,f.mgcv.out)
    }
    else{
      coef.mgcv_sel=rep(NA,ncol(V[[1]]))
      coef.mgcv_out=rep(NA,ncol(V[[2]]))
      sd.coef.mgcv_sel=sd.coef.mgcv_out=NA
    }
    result$coefficients.mgcv=list(selection=coef.mgcv_sel, outcome=coef.mgcv_out)
    result$sd.coefficients.mgcv=list(selection=sd.coef.mgcv_sel,outcome=sd.coef.mgcv_out)

  #------------------
  # Parametric frequentist estimation via package sampleSelection
    if (sampleSelection!="n"){
      cat("\nParametric estimation using package sampleSelection\n")
      require(sampleSelection)
      if (sampleSelection=="both"|sampleSelection=="2step")
        twostep=selection(selection=fakeformula.sel,outcome=fakeformula.out, data=data,method="2step")
        print(summary(twostep))
        result$twostepsd=sqrt(diag(vcov(twostep)))
        result$twostep=twostep$coefficients
      if (sampleSelection=="both"|sampleSelection=="ml"){
        temp=try(selection(selection=fakeformula.sel,outcome=fakeformula.out, data=data,method="ml"))
        print(temp)
        #print(try(summary(temp)))
        result$mlsd=try(sqrt(diag(vcov(temp))))
        result$ml=try(temp$estimate)
        rm(temp)
      }
    }


##############################################################
# Starting values and priors

  # Parametric effects
    # starting value of gamma[[2]] estimated via least squares, starting value of gamma[[1]] estimated via ML
    if (mgcv){
      gamma[[1]][1,]=coef.mgcv_sel
      gamma[[2]][1,]=coef.mgcv_out
      result$coefficients.glm=(glm(fakeformula.sel, data=data, family=binomial(link="probit"))$coef)[1:ncol(V[[1]])]
      result$coefficients.lm=(lm(fakeformula.out,data=data[response[[1]]==1,])$coef)[1:ncol(V[[2]])]
    }
    else{
      result$coefficients.glm=gamma[[1]][1,]=(glm(fakeformula.sel, data=data, family=binomial(link="probit"))$coef)[1:ncol(V[[1]])]
      result$coefficients.lm=gamma[[2]][1,]=(lm(fakeformula.out,data=data[response[[1]]==1,])$coef)[1:ncol(V[[2]])]
    }

    f.gamma[[1]]= V[[1]] %*% gamma[[1]][1,]
    f.gamma[[2]]= V[[2]] %*% gamma[[2]][1,]

  # Nonparametric effects
    # see initialize betas ==0

  # Starting values of the predictors
    # just linear effects, starting values of nonpar effects ==0
      eta[[1]]   = V[[1]] %*% gamma[[1]][1,]
      eta[[2]]   = V[[2]] %*% gamma[[2]][1,]

  # Components of the Covariancematrix
    # prior assumption for sigma12
      mu.sigma12_0 = prior.sigma12[1]   
      var.sigma12_0 = prior.sigma12[2]  

      # starting value of sigma12 implicitly ==0 by initialization 

   # sigma2_1 (denotes the second element in the variance of the error of the outcome equation
      # the first parameter remains constant, no updating necessary
        a.sigma2_1 = prior.sigma2_1[1] + sum(response[[1]]==1) #start + number of uncensored observations
      # starting value of sigma2_1
        sigma2_1[1]=sigma2_1.start

  # tau2
    a.tau.sp=list()
    a.tau.rw_0 = prior.tau[1]
    b.tau.rw_0 = prior.tau[2]
    a.tau.sp_0 = prior.tau.sp[1]
    b.tau.sp_0 = prior.tau.sp[2]
    # the first parameter remains constant, no updating necessary
      a.tau.rw = a.tau.rw_0 + qr(K.rw)$rank/2
      if (geo) {
        if (numSpatial_sel>0) a.tau.sp[[1]] = a.tau.sp_0 + qr(K.mrf[[1]])$rank/2
        if (numSpatial_out>0) a.tau.sp[[2]] = a.tau.sp_0 + qr(K.mrf[[2]])$rank/2
      }



############
#  GIBBS   #
############

p8=proc.time()
if (prof)  Rprof()

if (progressBar) pb = txtProgressBar(style=3)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Start of loop
for ( i in 2:(1+numBurnIn+numSamples) ){

if (i%%100==0) setTxtProgressBar(pb, i/(1+numBurnIn+numSamples))
#  if (i%%100==0){ cat(i,"")}
#  if (i%%3000==0) cat("\n")

  ###############
  # Sampling of latent variables
#Use package truncnorm or functions according to "R Programs for Computing Truncated Distributions" by Saralees Nadarajah and Samuel Kotz (2006)
    if (truncnorm){
      # censored observations
        response[[1]][z_aux==0] = rtruncnorm(n = length(response[[1]][z_aux==0]),
                                         a= -Inf, b = 0,
                                         mean = eta[[1]][z_aux==0],
                                         sd   = 1)

      # uncensored observations
        mu.z=  eta[[1]][z_aux==1] + (sigma12[i-1] / (sigma2_1[i-1] + sigma12[i-1]^2)) * (response[[2]] - eta[[2]])
        var.z= 1 - sigma12[i-1]^2 / (sigma2_1[i-1] + sigma12[i-1]^2)
        response[[1]][z_aux==1] = rtruncnorm(n= length(response[[1]][z_aux==1]),
                                         a= 0, b = Inf,
                                         mean = mu.z,
                                         sd   = sqrt(var.z))
    }
    else{
      # censored observations
        response[[1]][z_aux==0] = rtrunc(n = length(response[[1]][z_aux==0]),"norm",
                                         a= -Inf, b = 0,
                                         mean = eta[[1]][z_aux==0],
                                         sd   = 1)

      # uncensored observations
        mu.z=  eta[[1]][z_aux==1] + (sigma12[i-1] / (sigma2_1[i-1] + sigma12[i-1]^2)) * (response[[2]] - eta[[2]])
        var.z= 1 - sigma12[i-1]^2 / (sigma2_1[i-1] + sigma12[i-1]^2)
        response[[1]][z_aux==1] = rtrunc(n= length(response[[1]][z_aux==1]),"norm",
                                         a= 0, b = Inf,
                                         mean = mu.z,
                                         sd   = sqrt(var.z))
  }
    # Record Samples
      if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0) z.sum = z.sum + response[[1]]

  ###############
  # Sampling of sigma2_1
    # no updating of a.sigma2_1
    b.sigma2_1 = prior.sigma2_1[2] +  crossprod(sigma12[i-1] *(response[[1]][z_aux==1] - eta[[1]][z_aux==1]) - (response[[2]]-eta[[2]]))
    sigma2_1[i] = (rgamma(n=1, a.sigma2_1 /2, b.sigma2_1 /2))^(-1)

  ###############
  # Sampling of SIGMA12
    if (fix.sigma12) sigma12[i] = 0
    else{
      var.sigma12 = (var.sigma12_0^(-1) + sigma2_1[i]^(-1) * crossprod(response[[1]][z_aux==1] - eta[[1]][z_aux==1]))^(-1)
      mu.sigma12 = var.sigma12 * (var.sigma12_0^(-1) * mu.sigma12_0 + sigma2_1[i]^(-1) * crossprod((response[[1]][z_aux==1]-eta[[1]][z_aux==1]), response[[2]]-eta[[2]]))
      sigma12[i] = rnorm(n=1, mean=mu.sigma12, sd=sqrt(var.sigma12))
    }

  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Estimation of the coefficients

    cond.var[[1]]= 1 - sigma12[i]^2/(sigma2_1[i]+sigma12[i]^2)                       #conditional variance in the selection eq
    cond.var[[2]]= sigma2_1[i]         #= (sigma2_1[i]+sigma12[i]^2) - sigma12[i]^2       #conditional variance in the outcome eq

    offset[[1]]= sigma12[i]/(sigma2_1[i]+sigma12[i]^2) * (response[[2]] - eta[[2]])  #offset vector of the selection eq

    #======================
    # Parametric effects
      for (r in 1:2){
        # in the second loop, offset.outcome is updated
          if (r==2) {offset[[2]]=sigma12[i] * (response[[1]] - eta[[1]])}

        eta_tilde= eta[[r]] - f.gamma[[r]] # ==0 if numSplines_sel==0 or numSplines_out==0 

        # Updating of gamma[[r]]
          # Selection equation
            if (r==1){
              offset_tilde= offset[[1]] + eta_tilde[z_aux==1]
              VAR.gamma[[1]] = solve(cond.var[[1]]^(-1)* VV1[[1]]+ VV0[[1]]) #VV0[[2]] is null matrix-> last part discarded in outcome equation    # sampler A:VAR.gamma[[1]] = solve(cond.var_sel^(-1)* crossprod(V[[1]]))
              mu.gamma[[1]]= VAR.gamma[[1]]%*%((cond.var[[1]]^(-1)*t(V[[1]][z_aux==1,])%*%(response[[1]][z_aux==1] - offset_tilde)) + t(V[[1]][z_aux==0,])%*%(response[[1]][z_aux==0]-eta_tilde[z_aux==0]))
            }
          # Outcome equation
            if (r==2){
              offset_tilde= offset[[2]][z_aux==1] + eta_tilde
              VAR.gamma[[2]] = solve(cond.var[[2]]^(-1)* VV1[[2]]) #VV0[[2]] is null matrix           # sampler A:VAR.gamma[[1]] = solve(cond.var_sel^(-1)* crossprod(V[[1]]))
              mu.gamma[[2]]= VAR.gamma[[2]]%*%((cond.var[[2]]^(-1)*t(V[[2]])%*%(response[[2]] - offset_tilde)))
            }
          gamma[[r]][i,] = rmvnorm(n=1, mean=mu.gamma[[r]], sigma=VAR.gamma[[r]],method=method)


        f.gamma[[r]]= V[[r]] %*% gamma[[r]][i,]

        # Updating of the predictor
          eta[[r]]= eta_tilde + f.gamma[[r]]  # == eta[[1]] -V[[1]] %*%gamma[[1]][i-1,] + V[[1]] %*%gamma[[1]][i,]
     }
    offset[[1]]= sigma12[i]/(sigma2_1[i]+sigma12[i]^2) * (response[[2]] - eta[[2]])  # offset vector of the selection equation

    #======================
    # Nonparametric effects
      if (length(npvars) > 0){
        for (j in 1:length(npvars)){
          # selection or outcome equation?
            if (j> numSplines_sel+numSpatial_sel) r=2 else r=1

          # spatial effect or nonparametric effect?
            if ((numSpatial_sel>0 & j== numSplines_sel+1) | (numSpatial_out>0 & j== numSplines_sel+numSpatial_sel+numSplines_out+1))
              {K= K.mrf[[r]]; a.tau= a.tau.sp[[r]]; b.tau_0=b.tau.sp_0}
            else {K= K.rw; a.tau= a.tau.rw; b.tau_0=b.tau.rw_0}

          eta_tilde= eta[[r]] - f[[j]]  #== eta[[1]] - (X_sel[[j]] %*% beta_sel[[j]][i-1,] -i.constraint_j[i-1])

          # Updating of tau2
            #a.tau is only updated once
            b.tau= b.tau_0 + 1/2 * t(beta[[j]]) %*% K %*% beta[[j]]
            tau2 =(rgamma(n=1, a.tau, b.tau))^(-1)             

          # Updating of beta
            # Selection equation
              if (r==1){
                offset_tilde= offset[[1]] + eta_tilde[z_aux==1]
                  P= cond.var[[1]]^(-1) * XX1[[j]]+ XX0[[j]] + tau2^(-1) * K
                  L=chol(P)
                  nu= forwardsolve(L,(cond.var[[1]]^(-1) * t(X[[j]][z_aux==1,]) %*% (response[[1]][z_aux==1]- offset_tilde))+ t(X[[j]][z_aux==0,])%*%(response[[1]][z_aux==0]-eta_tilde[z_aux==0]), upper.tri = TRUE, transpose = TRUE)
                }
            # Outcome equation
              else{ #if (r==2)
                offset_tilde= offset[[2]][z_aux==1] + eta_tilde
                P= cond.var[[2]]^(-1) * XX1[[j]]+ tau2^(-1) * K        #XX0[[2]] ist nullmatrix-> entsprechender teil f?llt in outcome eq weg
                L=chol(P)
                nu= forwardsolve(L,(cond.var[[2]]^(-1) * t(X[[j]]) %*% (response[[2]]- offset_tilde)), upper.tri = TRUE, transpose = TRUE)
              }

            mu = backsolve(L,nu)
            # beta[i-1] -> beta[i]
              beta[[j]]= mu + solve(L,rnorm(dim(X[[j]])[2],0,1))

          f[[j]]= X[[j]] %*% beta[[j]]

          # centering (varying coefficients are not centered)
            if (byvars[j]=="NA"){  #no varying coefficient
              i.constraint_j = mean(f[[j]])
              f[[j]]= f[[j]] - i.constraint_j #== f[[j]]= scale(f[[j]],scale=F)
              # compensation in intercept
                eta_tilde = eta_tilde - f.gamma[[r]]    
                # Updating of "f.gamma" with the updated intercept
                  gamma[[r]][i,1] = gamma[[r]][i,1] + i.constraint_j
                  f.gamma[[r]]= V[[r]] %*% gamma[[r]][i,]
                eta_tilde = eta_tilde + f.gamma[[r]]
            }
          # Updating of eta[[r]]
            eta[[r]]= eta_tilde + f[[j]] #== eta[[r]]= eta[[r]] - (X[[j]] %*%beta[[j]][i-1,] - i.constraint_j[i-1]) + X[[j]] %*% beta[[j]][i,] - i.constraint_j[i]

          # Record Samples
            if (i>(numBurnIn+1) & (i-(numBurnIn+1))%%thin==0){
              if (byvars[j]=="NA"){
                f.sum[[j]]=f.sum[[j]]+f[[j]]
                f.rec[[j]][,fnr[[j]]]=f[[j]]
                if (!is.null(x.pred[[j]])){
                  f.pred[[j]]= X.pred[[j]] %*% beta[[j]] - i.constraint_j
                  f.pred.sum[[j]]=f.pred.sum[[j]]+f.pred[[j]]
                  f.pred.rec[[j]][,fnr[[j]]]=f.pred[[j]]
                }
              } 
              if (record.npcoef| byvars[j]!="NA")
                beta.rec[[j]][fnr[[j]],]=beta[[j]]

              fnr[[j]]=fnr[[j]]+1
            }

          # When all nonparametric effects in the selection equation are estimated, offset.outcome is updated
            if (j==(numSplines_sel+numSpatial_sel)) offset[[2]]=sigma12[i] * (response[[1]]- eta[[1]])
        }
      }


  #%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  # Unstandardized deviance
    Dev[i]=log(2*pi) +crossprod(response[[1]]-eta[[1]])  +  log(2*pi*sigma2_1[i])+sigma2_1[i]^(-1)*crossprod(sigma12[i]*(response[[1]][z_aux==1]-eta[[1]][z_aux==1])-(response[[2]]-eta[[2]]))

  if (i==51)  {cat("\nEstimated time to end: ", ((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%/%60,"min",round(((proc.time()[3]-p8[3])*(numBurnIn+numSamples)/50)%%60,0),"sec\n");cat("\nProgress:\n ")}
}
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# end of loop
if (prof) Rprof(NULL)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Calculation of final estimators
  #*********************
  # Covariance components
    # Discarding burn-in and thinning
      result$sigma12.sample = sigma12[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin) ]
      result$sigma2_1.sample = sigma2_1[seq((numBurnIn+2),(numBurnIn+numSamples+1),thin) ]

    result$sigma12 = mean(result$sigma12.sample)
    result$sigma2_1     = mean(result$sigma2_1.sample)

    result$cov   = matrix(c( 1,            result$sigma12,
                             result$sigma12, result$sigma2_1+result$sigma12^2 ), ncol=2)

    result$corr =     mean(result$sigma12.sample/sqrt(result$sigma2_1.sample+ result$sigma12.sample^2),rm.na=T)


    result$ci.sigma12= quantile(result$sigma12.sample, probs= c(0.025, 0.05, 0.5, 0.95, 0.975))
    result$ci.sigma22= quantile(result$sigma2_1.sample+ result$sigma12.sample^2, probs= c(0.025, 0.05, 0.5, 0.95, 0.975))
    result$ci.corr=    quantile(result$sigma12.sample/sqrt(result$sigma2_1.sample+ result$sigma12.sample^2), probs= c(0.025, 0.05, 0.5, 0.95, 0.975))

    result$sd.sigma12=  sqrt(var(result$sigma12.sample))
    result$sd.sigma22=  sqrt(var(result$sigma2_1.sample+ result$sigma12.sample^2))
    result$sd.corr=     sqrt(var(result$sigma12.sample/sqrt(result$sigma2_1.sample+ result$sigma12.sample^2)))

  #*********************
  # Parametric effects
    # Discarding burn-in and thinning
      gamma_sel.sample = as.data.frame(gamma[[1]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])
      gamma_out.sample = as.data.frame(gamma[[2]][seq((numBurnIn+2),(numBurnIn+numSamples+1),thin), ,drop=F])
      result$coefficients.sample=list(selection=gamma_sel.sample,outcome =gamma_out.sample)

    gamma_sel.mean   = colMeans(gamma_sel.sample)
    gamma_out.mean   = colMeans(gamma_out.sample)
    names(gamma_sel.mean)= dimnames(gamma_sel.sample)[[2]]= g_sel
    names(gamma_out.mean)= dimnames(gamma_out.sample)[[2]]= g_out
    result$coefficients=list(selection=gamma_sel.mean,outcome=gamma_out.mean)

    sd.sel=apply(gamma_sel.sample,2, function(x) sqrt(var(x)))
    sd.out=apply(gamma_out.sample,2, function(x) sqrt(var(x)))
    result$sd.coefficients=list(selection=sd.sel,outcome=sd.out)

    # Quantiles
      ci_gamma.sel = apply(gamma_sel.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      ci_gamma.out = apply(gamma_out.sample, MARGIN=2, quantile, probs= c(0.025, 0.5, 0.975))
      result$ci.coefficients=list(selection=ci_gamma.sel,outcome=ci_gamma.out)

  #*********************
  # Nonparametric effects
     ci.f.pred= result$ci.f= result$f.by= result$f=result$ci.npcoef=result$npcoef=  list()
    if (length(npvars)>0){
    result$eq=numeric()
      if (numSplines_sel>0|numSpatial_sel>0) result$eq=rep(1,(numSplines_sel+numSpatial_sel))
      if (numSplines_out>0|numSpatial_out>0) result$eq=c(result$eq,rep(2,numSplines_out+numSpatial_out))
      for (j in 1:length(npvars)){
        if (length(beta.rec[[j]])>1){
          result$npcoef[[j]]   = colMeans(beta.rec[[j]])
          result$ci.npcoef[[j]]= apply(beta.rec[[j]], MARGIN=2, quantile, probs= c(0.025, 0.10, 0.90, 0.975))
        }
        else result$npcoef[[j]] = result$ci.npcoef[[j]]= 0

        if (length(f.rec[[j]])>1){
          result$f[[j]]= f.sum[[j]] /(numSamples%/%thin)
          result$ci.f[[j]]= t(apply(f.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          if (!is.null(x.pred[[j]])){
            f.pred[[j]]=f.pred.sum[[j]] /(numSamples%/%thin)
            ci.f.pred[[j]]= t(apply(f.pred.rec[[j]], MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          }
        }
        else result$f[[j]] = result$ci.f[[j]]= f.pred[[j]]=0

        if (byvars[j]!="NA"){
          result$f[[j]]=X[[j]]%*%result$npcoef[[j]]
          result$f.by[[j]]=Xby[[j]]%*%result$npcoef[[j]]
          temp.sample=apply(beta.rec[[j]], 1, function(x) Xby[[j]]%*%x)
          result$ci.f[[j]]= t(apply(temp.sample, MARGIN=1, quantile, probs= c(0.025, 0.10, 0.90, 0.975)))
          rm(temp.sample)
        }
        else result$f.by[[j]]=0
      }
      names(result$npcoef) = names(result$f) = names(result$f.by) = names(result$ci.npcoef)= names(result$ci.f)  = npvars
      if (!all(unlist(lapply(x.pred,is.null)))){
        names(f.pred)=npvars[byvars=="NA"]
        for (j in 1:length(npvars)) if (byvars[j]!="NA") names(f.pred)= paste(byvars[j],"*",npvars[j],sep="") 
      }
      for (j in 1:length(npvars)) if (byvars[j]!="NA") names(result$f)[j]=paste(byvars[j],"*",npvars[j],sep="")
    }


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# "CI's" for spatial effects
  if (numSpatial_sel>0) {
    pcat80.sel=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_sel+1]][,2]), t(result$ci.f[[numSplines_sel+1]][,3]))
    pcat95.sel=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      t(result$ci.f[[numSplines_sel+1]][,1]), t(result$ci.f[[numSplines_sel+1]][,4]))
  }
  else pcat80.sel=pcat95.sel=NULL
  if (numSpatial_out>0) {
    pcat80.out=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_sel+numSpatial_sel+numSplines_out+1]][,2],result$ci.f[[numSplines_sel+numSpatial_sel+numSplines_out+1]][,3])
    pcat95.out=mapply(FUN=function(x,y) {if (x<0 & y<0) return(-1)
                                         if ((x<=0 & y>=0)|(x>=0 & y<=0)) return(0)
                                         if (x>0 & y>0) return(1)},
                      result$ci.f[[numSplines_sel+numSpatial_sel+numSplines_out+1]][,1],result$ci.f[[numSplines_sel+numSpatial_sel+numSplines_out+1]][,4])
  }
  else pcat80.out=pcat95.out=NULL

  result$pcat80=list(selection=pcat80.sel,outcome=pcat80.out)
  result$pcat95=list(selection=pcat95.sel,outcome=pcat95.out)

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Fitted values / eta
  # Selection equation
    fitted.sel= V[[1]]%*%result$coefficients$selection
    if (numSplines_sel>0 |numSpatial_sel>0){
      for (j in 1:(numSplines_sel+numSpatial_sel)) 
        fitted.sel = fitted.sel + result$f[[j]]
    }
  # Outcome equation
    fitted.out= V[[2]]%*%result$coefficients$outcome
    if (numSplines_out>0 |numSpatial_out>0){
      for (j in 1:(numSplines_out+numSpatial_out))
        fitted.out=fitted.out + result$f[[j + (numSplines_sel+numSpatial_sel)]]
    }

  result$fitted.values=list(selection=fitted.sel,outcome=fitted.out)

# DIC
  # mean of the latent variable in selection equation
    z.mean = z.sum / (numSamples%/%thin)
 
  Dev.sample = Dev[seq((numBurnIn+2), (numBurnIn+numSamples+1), thin)]
  Dev.mean = mean(Dev.sample)

  Dev.post = log(2*pi) + crossprod(z.mean - fitted.sel)   +   log(2*pi*result$sigma2_1) + result$sigma2_1^(-1) * crossprod(result$sigma12*(z.mean[z_aux==1]-fitted.sel[z_aux==1]) - (response[[2]]-fitted.out))
  result$DIC = 2*Dev.mean - Dev.post


#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Posterior probabilities for parametric effects
  pl0.sel= apply(gamma_sel.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.sel= apply(gamma_sel.sample,2,function(x) 1/length(x)*sum(x>0))
  pl0.out= apply(gamma_out.sample,2,function(x) 1/length(x)*sum(x<0))
  pu0.out= apply(gamma_out.sample,2,function(x) 1/length(x)*sum(x>0))

  result$pvalues.onesided=list(selection=data.frame(cbind(pl0.sel,pu0.sel),row.names=g_sel),outcome=data.frame(cbind(pl0.out,pu0.out),row.names=g_out))
  result$pvalues.twosided=list(selection=apply(result$pvalues.onesided$selection,1,function(x) 2*min(x)),outcome=apply(result$pvalues.onesided$outcome,1,function(x) 2*min(x)))

p9= proc.time()
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


  result$npcoef.sample=beta.rec
  if (return.f) result$f.sample= f.rec
  if (!all(unlist(lapply(x.pred,is.null)))){
   result$f.pred=f.pred
   result$ci.f.pred=ci.f.pred
  }
  result$responses=list(z_aux,response[[2]],response[[1]])
  names(result$responses)=c(response.sel,response.out,paste(response.sel,".aug",sep=""))


#    result$coefficients=list(ssm=list(selection=gamma_sel.mean,outcome=gamma_out.mean),glm=list(selection=coefficients.glm),lm=list(outcome=coefficients.lm),mgcv=list(selection=coef.mgcv_sel, outcome=coef.mgcv_out)
#  result$X=X,
#  result$pcovariates=V
#  result$npcovariates=npcovariates
#  result$bycovariates=byvars.list
  result$covariates=list(parametric=V,nonparametric=npcovariates,by=byvars.list)
  result$x.pred=x.pred
  result$formulas=list(selection=selection,outcome=outcome)
  result$numSplines=list(selection=numSplines_sel,otucome=numSplines_out)
  result$numSpatial=list(selection=numSpatial_sel,outcome=numSpatial_out)
  result$sp=list(selection=sp_sel,outcome=sp_out)
  result$numBurnIn=numBurnIn
  result$numSamples=numSamples
  result$thin=thin

  

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# OUTPUT
if (progressBar) close(pb)


cat("\n\nTotal time elapsed\n");print(p9-p0)
class(result)=c("bayesIV","ssm")

summary(result)

# PLOTS
  #----------------------------------------------
  # Plot sampling paths of parametric effects
    if (plotpaths & (ncol(V[[1]])>0 | ncol(V[[2]])>0))
      plotPaths(cbind(gamma[[1]], gamma[[2]]), names=c(g_sel,g_out),numBurnIn=numBurnIn) # , output=ifelse(output=="window",yes="window",no=paste(output,date,"SamplingPaths.pdf")),style=plotpaths.style


if (!return.samples) result$sigma12.sample=result$sigma2_1.sample=result$coefficients.sample=result$npcoef.sample= NULL

invisible(result)
}

# Values
#f list of estimated functions
#f.pred estimated functions corresponding to x.pred
#cov covariance matrix

#pcoef parametric coefficients
#sd.pcoef std.dev. of parametric coeffs