\name{bayesIVgauss}
\alias{bayesIVgauss}
\title{Fitting Bayesian Semiparametric Instrumental Variable Regression models with bivarariate Gaussian errors}
\description{
Fits a Bayesian semiparametric instrumental variable regression model with bivariate normal error distribution.
Inference is fully Bayesian. Nonparametric effects are modeled with P-splines. Also geoadditive effects and
varying coefficients can be fitted.
}
\usage{
bayesIVgauss(first, second, random = list(NULL, NULL), data,
  numBurnIn = 2000, numSamples = 20000, thin = 20,
  numKnots = 40, degree = 3, rw.order = 2, %knotchoice = "equidistant",
  contrasts.first = NULL, contrasts.second = NULL, graph = NULL, x.pred = NULL, SCALE = T,
  prior.cov = list(s.Sigma = 3, S.Sigma = 0.2 * diag(2)),
  prior.random = c(0.001, 0.001), 
  prior.tau = c(0.001, 0.001), prior.tau.sp = c(0.001, 0.001),
  seed = NULL, scbs = F, scb.level = 0.95, scb.method = "TK",
  record.npcoef = F, return.samples = T, return.f = T,
  mgcv = F, plotpaths = F, progressBar = T, method = "chol")

}
\arguments{
\item{first, second}{the formulas of first and second equation.
Specification of formulas is similar to package \code{mgcv}, with an additional possibility to fit spatial
    effects specified with \code{sp(.)} (smooth functions
    are specified with \code{s(.)} and varying coefficients with
    \code{s(., by=.)}). If nonparametric effects are included, an
    intercept is obligatory, otherwise exclusion of the intercept with
    -1 and +0, as usual. Numeric variables have to be specified first, categorical variables last.}
\item{random}{list of cluster IDs for random intercepts in the two equations. Must be given as factors.}
\item{data}{a data frame containing the variables in the model. Might cause problems if data frame is not called "data" in some instances.}
\item{graph}{a \code{\link{BayesX}} graph object containing information of neighborhood structures for spatial effects.}
\item{contrasts.first, contrasts.second}{optional lists for the specification of contrasts in the case of catergorical covariates in first and second equation, respectively. See \code{contrasts.arg} of \code{\link{model.matrix.default}}.}
\item{numBurnIn, numSamples}{number of iterations in burn-in period that will be discarded and number of samples that will be used for estimation, respectively.}
\item{thin}{the degree of thinning. If 1, every sample after the burn-in period is used.}
\item{numKnots}{number of knots for nonparametric effects.}
\item{degree}{the degree of the B-spline basis functions according to function bs() in package \code{splines}, default is 3 for cubic splines.}
\item{rw.order}{the order of the random walk penalty matrices of smooth functions. }
\item{x.pred}{(optional) list of vectors of covariates for which predicted function values are returned. }
\item{SCALE}{TRUE to scale responses. Recommended.}
\item{prior.cov}{
%covariance matrix of the prior bivariate Gaussian distribution
\code{s.Sigma} and \code{S.Sigma} for the hyperparameters of the inverted Wishart on the residual covariance matrix.
}
\item{prior.random}{shape and scale parameters for the inverse Gamma prior on the variances of possible random effects}
\item{prior.tau}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of curves.      }
\item{prior.tau.sp}{shape and scale parameters for the inverse Gamma prior on the variance parameters which influence the smoothness of spatial effects.}
\item{seed}{a seed for set.seed (optional).}
\item{record.npcoef}{record samples of the regression coefficients of nonparametric effects? Returned only if return.samples=TRUE.  }
\item{return.samples}{return matrices of samples of all estimates (including components of the covariance matrix, nonparametric (only if record.beta=T) and parametric estimates)? }
\item{return.f}{Specify if fitted values for every smooth effect of each Gibbs iteration are to be returned. }
\item{mgcv}{Specify if univariate estimation using package mgcv is included for comparison. Spatial effects are discarded.}
\item{scbs}{if TRUE, simultaneous credible bands are computed}
\item{scb.level}{level of simultaneous credible bands}
\item{scb.method}{method for simultaneous credible band: "TK" (Thomas Kneib), "CC" (Ciprian Crainiceanu) or "JB" (Besag)}
\item{method}{}
\item{plotpaths}{logical, if TRUE the sampling paths are plotted  }
\item{progressBar}{show txtProgressBar during simulation}
%\item{output}{  Device on which plots are printed. Default is "window", otherwise pdf files are written with file name starting with what is given here.}
%\item{knotchoice}{Specify the way knots are located. Either "equidistant" or "quantiles".}
%\item{add.mgcv}{  Add smooth functions estimated by \code{mgcv} to the plots of nonparametric effects. Doesn't work with varying coefficients.       }
%\item{plotnp}{  Logical, if TRUE nonparametric effects are plotted  }
%\item{plotmaps}{  Logical, if TRUE the maps(s) with spatial effect(s) is/are plotted}
%\item{cols}{Color scheme to be employed for plotting spatial maps. Could be either a vector of colors or one out of the following pre-defined schemes: hcl, hsv, grey}
%\item{plotacf}{  Plot autocorrelation functions? If nonparametric effects are included autocorrelation functions of the first and 10th knot are plotted, but only if record.beta==T.              }
%\item{lag}{  Specify the lag of the autocorrelation functions.    }
%  \item{plotmgcv}{}
%  \item{plotpaths.style}{}
}
\details{
Fits a semiparametric instrumental variable model that allows to correct for endogeneity bias in
regression models with one endogenous regressor where the covariate effects enter with unknown functional form.
Bias correction relies on a simultaneous equations specification assuming a bivariate Gaussian joint error distribution.
The structural and instrumental variable equation are specified based on geoadditive predictors comprising penalised splines
for nonlinear effects of continuous covariates, Gaussian Markov random fields for spatial effects. Inference is fully Bayesian, employing efficient
Markov Chain Monte Carlo simulation techniques. The resulting posterior samples allow the construction of simultaneous credible bands for the nonparametric effects.
A further advantage includes the automatic smoothing parameter selection.

The output includes resulting estimates, as well as the corresponding credible intervals and
posterior probabilities.

%For model first the DIC (Deviance Information Criterion) which is based on the joint likelihood, is calculated.
It is recommended to choose large numbers of numBurnIn, numSamples and thin and to check sample autocorrelations when instruments are weak.
}
\value{
\item{coefficients.sample, npcoef.sample}{MCMC samples of the parameters after the burn-in period and after thinning.   }
\item{f.sample}{MCMC samples of the nonparametric functions.   }
\item{f, f.mgcv}{Function values evaluated at the designpoints for the simultaneous equations model and separate univariate regressions using \code{\link[mgcv]{gam}}.   }
\item{f.by}{Function values of functions with varying coefficients evaluated at the designpoints}
\item{ci.f}{The corresponding pointwise credible intervals.   }
\item{f.pred, f.mgcv.pred, ci.f.pred}{Predicted function values evaluated at points given by \code{x.pred} for the joint model and separate univariate regressions using \code{\link[mgcv]{gam}} and the corresponding credible intervals.   }
\item{pcat80, pcat95}{For the visualisation of 80\% and 95\% posterior probabilities of geoadditive effects. Regions with significantly positive, insignificant and significantly negative values are coded with +1, 0 and -1, respectively.   }
\item{coefficients,coefficients.mgcv,coefficients.glm,coefficients.lm}{Coefficients in the simultaneous equations model, separate univariate regressions obtained by \code{\link[mgcv]{gam}} and separate univariate parametric regressions obtained by \code{\link{lm}}.   }
\item{sd.coefficients,sd.coefficients.mgcv}{The standard deviations of the coefficients }
\item{ci.coefficients}{ 90\% and 95\% credible intervals of the parametric effects }
\item{npcoef,ci.npcoef}{coefficients and credible intervals of the coefficients of nonparametric effects}
\item{pvalues.onesided}{Onesided Bayesian p-values.}
\item{pvalues.twosided}{Two-sided Bayesian p-values.}
\item{responses}{The responses}
\item{covariates}{The included covariates}
\item{x.pred}{the grid points}
\item{fitted.values}{The fitted values.}
\item{formulas, numSplines, numSpatial, numBurnIn, numSamples, thin}{Arguments given in the function call.   }
%\item{cov, sigma12, sigma2_1, corr}{The covariance matrix of the errors, the associated covariance sigma12, conditional variance $sigma_{2|1}^2$ and correlation.}
%\item{ci.sigma12, ci.sigma22, ci.corr}{90\% and 95\% credible intervals of the covariance, the error in the second equation and the correlation between the errors based on the respective quantiles of the samples. }
%\item{sd.sigma12, sd.sigma22, sd.corr}{The respective standard deviations.}
%\item{DIC}{Deviance Information Criterion for model selection.}
}
\references{
%% ~put references to the literature/web site here ~
}
\author{Manuel Wiesenfarth}
\note{
%%  ~~further notes~~
}

\seealso{
\code{\link{bayesIV}} for estimation assuming a flexible error distribution,  \code{\link{am}} for estimation without bias correction; package \code{\link{BayesX}} for description of spatial effects related functions.

%% ~~objects to See Also as \code{\link{help}}, ~~~
}
\examples{
# see bayesIV()
}